/**
 * File: /src/components/Image/index.tsx
 * Project: @native-theme-ui/core
 * File Created: 13-06-2022 00:51:44
 * Author: Clay Risser
 * -----
 * Last Modified: 04-09-2022 07:50:57
 * Modified By: Clay Risser
 * -----
 * Risser Labs LLC (c) Copyright 2021 - 2022
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import React, { FC, ComponentProps, useState } from "react";
import { Sx, styled } from "dripsy";
import {
  Image as RNImage,
  ImageSourcePropType,
  StyleSheet,
  ImageURISource,
  ImageResizeMode,
  ImageStyle,
} from "react-native";
import { useCalculatedSx } from "@risserlabs/auto-contrast";
import { Box } from "../Box";
import { ImageProps, useSplitProps } from "./props";

const defaultStyle: Sx = {
  height: "auto",
  maxWidth: "100%",
  width: "100%",
};

const StyledImage = styled(RNImage, { themeKey: "images" })(defaultStyle);

const StyledOuterBox = styled(Box)({
  position: "relative",
  overflow: "hidden",
});

export const Image: FC<ImageProps> = (props: ImageProps) => {
  const [imageRatio, setImageRatio] = useState<number>();
  const sx = useCalculatedSx({ ...props.sx });
  const style = StyleSheet.flatten(props.style || {}) as ImageStyle;
  style.resizeMode = getResizeMode(
    sx,
    (style as { objectFit?: string }).objectFit,
    style.resizeMode
  );
  delete (style as { objectFit?: string }).objectFit;
  delete sx.objectFit;
  const {
    baseProps,
    baseStyle,
    baseSx,
    innerStyle,
    innerSx,
    motiPressableProps,
  } = useSplitProps(props, sx);
  const isMotiPressable = !!Object.keys(motiPressableProps).length;

  const styledImageProps = { ...baseProps } as ComponentProps<
    typeof StyledImage
  > & {
    src?: string | ImageSourcePropType;
  };
  if (typeof props.src === "string") {
    styledImageProps.source = { uri: props.src };
  } else {
    styledImageProps.source = props.src;
  }
  delete styledImageProps.src;

  const calculateHeight =
    style.height === "auto" ||
    (typeof style.height === "undefined" &&
      (typeof sx.height === "undefined" || sx.height === "auto"));

  const uri =
    typeof props.src === "string"
      ? props.src
      : (props.src as ImageURISource)?.uri;
  if (uri && !imageRatio && calculateHeight) {
    RNImage.getSize(uri, (width: number, height: number) => {
      setImageRatio(width / height);
    });
  }

  if (calculateHeight && imageRatio) {
    return (
      <StyledOuterBox {...motiPressableProps} sx={baseSx} style={baseStyle}>
        <Box
          style={{
            height: 0,
            paddingBottom: `${100 / imageRatio}%`,
            width: "100%",
          }}
        />
        <StyledImage
          {...styledImageProps}
          sx={innerSx}
          style={{
            ...innerStyle,
            position: "absolute",
            top: 0,
            right: 0,
            bottom: 0,
            left: 0,
          }}
        />
      </StyledOuterBox>
    );
  }

  if (isMotiPressable) {
    return (
      <StyledOuterBox
        {...motiPressableProps}
        sx={baseSx}
        style={{ ...baseStyle }}
      >
        <StyledImage
          {...styledImageProps}
          sx={innerSx}
          style={{
            ...innerStyle,
            overflow: "visible",
            width: "100%",
            height: "100%",
          }}
        />
      </StyledOuterBox>
    );
  }

  return <StyledImage {...styledImageProps} sx={sx} style={style} />;
};

function getResizeMode(
  sx: Sx,
  objectFit?: string,
  resizeMode?: ImageResizeMode
): ImageResizeMode {
  if (resizeMode) return resizeMode;
  if (objectFit === "fill") return "stretch";
  if (objectFit === "cover" || objectFit === "contain") {
    return objectFit;
  }
  if (sx.objectFit === "fill") return "stretch";
  if (sx.objectFit === "cover" || sx.objectFit === "contain") {
    return sx.objectFit;
  }
  return "contain";
}

export type { ImageProps };
